@@ -238,21 +238,41 @@ def getLowerParams(param):
     return ret
 
 
-def get_all_params(param, body):
+def get_all_params(request):
     """
-    Combine parameters from GET and POST requests
-    """
-    return_param = {}
-    for key in param.keys():
-        return_param[key] = unquote(param[key])
+    Retrieve all parameters from a request, no matter if these are GET or POST requests
+    or parameters are contained as viewargs like the serial in DELETE /token/<serial>
 
-    # In case of serialized JSON data in the body, add these to the values.
-    try:
-        json_data = json.loads(to_unicode(body))
-        for k, v in json_data.items():
-            return_param[k] = v
-    except Exception as exx:
-        log.debug("Can not get param: {0!s}".format(exx))
+    :param request: The flask request object
+    """
+    param = request.values
+    body = request.data
+    return_param = {}
+    if param:
+        log.debug(u"Update params in request {0!s} {1!s} with values.".format(request.method,
+                                                                              request.base_url))
+        # Add the unquoted HTML and form parameters
+        return_param = {key: unquote(value) for (key, value) in param.items()}
+
+    if request.json:
+        log.debug(u"Update params in request {0!s} {1!s} with JSON data.".format(request.method,
+                                                                                 request.base_url))
+        # Add the original JSON data
+        return_param.update(request.json)
+    elif body:
+        # In case of serialized JSON data in the body, add these to the values.
+        try:
+            json_data = json.loads(to_unicode(body))
+            for k, v in json_data.items():
+                return_param[k] = v
+        except Exception as exx:
+            log.debug("Can not get param: {0!s}".format(exx))
+
+    if request.view_args:
+        log.debug(u"Update params in request {0!s} {1!s} with view_args.".format(request.method,
+                                                                                 request.base_url))
+        # We add the unquoted view_args
+        return_param.update({key: unquote(value) for (key, value) in request.view_args.items()})
 
     return return_param
 
@@ -313,7 +333,7 @@ def verify_auth_token(auth_token, required_role=None):
                     log.warning(u"Unsupported JWT algorithm in PI_TRUSTED_JWT.")
             except jwt.DecodeError as err:
                 log.info(u"A given JWT definition does not match.")
-            except jwt.ExpiredSignature as err:
+            except jwt.ExpiredSignatureError as err:
                 # We have the correct token. It expired, so we raise an error
                 raise AuthError(_("Authentication failure. Your token has expired: {0!s}").format(err),
                                 id=ERROR.AUTHENTICATE_TOKEN_EXPIRED)
@@ -324,7 +344,7 @@ def verify_auth_token(auth_token, required_role=None):
         except jwt.DecodeError as err:
             raise AuthError(_("Authentication failure. Error during decoding your token: {0!s}").format(err),
                             id=ERROR.AUTHENTICATE_DECODING_ERROR)
-        except jwt.ExpiredSignature as err:
+        except jwt.ExpiredSignatureError as err:
             raise AuthError(_("Authentication failure. Your token has expired: {0!s}").format(err),
                             id=ERROR.AUTHENTICATE_TOKEN_EXPIRED)
     if wrong_username:
