@@ -64,6 +64,8 @@ from privacyidea.lib import queue
 from privacyidea.lib.log import DEFAULT_LOGGING_CONFIG
 from privacyidea.config import config
 from privacyidea.models import db
+from privacyidea.lib.crypto import init_hsm
+
 
 ENV_KEY = "PRIVACYIDEA_CONFIGFILE"
 
@@ -87,7 +89,7 @@ class PiResponseClass(Response):
 
 def create_app(config_name="development",
                config_file='/etc/privacyidea/pi.cfg',
-               silent=False):
+               silent=False, init_hsm=False):
     """
     First the configuration from the config.py is loaded depending on the
     config type like "production" or "development" or "testing".
@@ -103,6 +105,8 @@ def create_app(config_name="development",
     :param silent: If set to True the additional information are not printed
         to stdout
     :type silent: bool
+    :param init_hsm: Whether the HSM should be initialized on app startup
+    :type init_hsm: bool
     :return: The flask application
     :rtype: App object
     """
@@ -222,10 +226,14 @@ def create_app(config_name="development",
         # header the browser transmits.  We support de/fr/en in this
         # example.  The best match wins.
         return request.accept_languages.best_match(['de', 'nl',
-                                                    'fr', 'it', 'es', 'en'])
+                                                    'fr', 'cs', 'it', 'es', 'en'])
 
     queue.register_app(app)
 
+    if init_hsm:
+        with app.app_context():
+            init_hsm()
+
     logging.debug(u"Reading application from the static folder {0!s} and "
                   u"the template folder {1!s}".format(app.static_folder, app.template_folder))
 
