@@ -5,6 +5,8 @@
 #  License: AGPLv3
 #  contact: http://www.privacyidea.org
 #
+#  2022-02-03 Cornelius Kölbel <cornelius.koelbel@netknights.it>
+#             Add verified enrollment
 #  2018-06-06 Michael Becker <michael.becker@hs-niederrhein.de>
 #             Add get_setting_type to make hotp.hashlib public and
 #             therefore recognised in token enrollment with role user.
@@ -53,7 +55,7 @@ from privacyidea.lib.config import get_from_config
 from privacyidea.lib.tokenclass import (TokenClass,
                                         TWOSTEP_DEFAULT_DIFFICULTY,
                                         TWOSTEP_DEFAULT_CLIENTSIZE,
-                                        TOKENKIND)
+                                        TOKENKIND, ROLLOUTSTATE)
 from privacyidea.lib.log import log_with
 from privacyidea.lib.apps import create_google_authenticator_url as cr_google
 from privacyidea.lib.error import ParameterError
@@ -77,12 +79,19 @@ keylen = {'sha1': 20,
           'sha512': 64
           }
 
+VERIFY_ENROLLMENT_MESSAGE = _("Please enter a valid OTP value of the new token.")
+
 
 class HotpTokenClass(TokenClass):
     """
     hotp token class implementation
     """
 
+    previous_otp_offset = 1
+
+    # The HOTP token provides means to verify the enrollment
+    can_verify_enrollment = True
+
     @staticmethod
     def get_class_type():
         """
@@ -140,11 +149,6 @@ class HotpTokenClass(TokenClass):
                            'desc': _("The user may only have this maximum number of active HOTP tokens assigned."),
                            'group': GROUP.TOKEN
                        },
-                       'yubikey_access_code': {
-                           'type': 'str',
-                           'desc': _("The Yubikey access code used to "
-                                     "initialize Yubikeys.")
-                       },
                        'hotp_2step_clientsize': {
                            'type': 'int',
                            'desc': _("The size of the OTP seed part contributed "
@@ -368,6 +372,23 @@ class HotpTokenClass(TokenClass):
                      get_from_config("hotp.hashlib", u'sha1')
         return hashlibStr
 
+    def _calc_otp(self, counter):
+        """
+        Helper function to calculate the OTP value for the given counter
+        :param counter: The OTP counter
+        :return: OTP value as string
+        """
+        otplen = int(self.token.otplen)
+        secretHOtp = self.token.get_otpkey()
+        hmac2Otp = HmacOtp(secretHOtp,
+                           self.get_otp_count(),
+                           otplen,
+                           self.get_hashlib(self.hashlib))
+
+        otpval = hmac2Otp.generate(counter=counter,
+                                   inc_counter=False,
+                                   do_truncation=True)
+        return otpval
 
     @log_with(log)
     @check_token_locked
@@ -446,7 +467,7 @@ class HotpTokenClass(TokenClass):
         return res
 
     @log_with(log)
-    def is_previous_otp(self, otp, window=10):
+    def is_previous_otp(self, otp):
         """
         Check if the OTP values was previously used.
 
@@ -454,12 +475,17 @@ class HotpTokenClass(TokenClass):
         :param window:
         :return:
         """
-        res = False
-        r = self.check_otp_exist(otp, window=window, symetric=True,
-                                 inc_counter=False)
-        if 0 <= r < self.token.count:
-            res = True
-        return res
+        counter = int(self.get_otp_count())
+        if counter > 0:
+            previous_otp = self._calc_otp(counter - self.previous_otp_offset)
+            res = previous_otp == otp
+            if res:
+                log.info("Previous OTP used again. "
+                         "Serial {0!s} with counter {1!s}.".format(self.token.serial, counter))
+            return res
+        else:
+            # The internal counter is 0, the token was not used, yet.
+            return False
 
     @log_with(log)
     def _autosync(self, hmac2Otp, anOtpVal):
@@ -753,3 +779,29 @@ class HotpTokenClass(TokenClass):
         if len(l) >= 5:
             params["counter"] = int(l[4].strip())
         return params
+
+    def prepare_verify_enrollment(self):
+        """
+        This is called, if the token should be enrolled in a way, that the user
+        needs to provide a proof, that the server can verify, that the token
+        was successfully enrolled. E.g. with HOTP tokens the user might need to provide
+        a correct OTP value.
+
+        The returned dictionary is added to the response in "detail" -> "verify".
+
+        :return: A dictionary with information that is needed to trigger the verification.
+        """
+        return {"message": VERIFY_ENROLLMENT_MESSAGE}
+
+    def verify_enrollment(self, verify):
+        """
+        This is called during the 2nd step of the verified enrollment.
+        This method verifies the actual response from the user.
+        Returns true, if the verification was successful.
+
+        :param verify: The response given by the user
+        :return: True
+        """
+        r = self.check_otp(verify)
+        log.debug("Enrollment verified: {0!s}".format(r))
+        return r >= 0
