@@ -19,7 +19,7 @@
 # You should have received a copy of the GNU Affero General Public
 # License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-__doc__ = """The pushtoken sends a push notification via firebase
+__doc__ = """The pushtoken sends a push notification via Firebase service
 to the registered smartphone.
 The token is a challenge response token. The smartphone will sign the challenge
 and send it back to the authentication endpoint. 
@@ -46,7 +46,7 @@ from privacyidea.lib.policy import SCOPE, ACTION, GROUP, get_action_values_from_
 from privacyidea.lib.log import log_with
 from privacyidea.lib import _
 
-from privacyidea.lib.tokenclass import TokenClass, TOKENMODE
+from privacyidea.lib.tokenclass import TokenClass, AUTHENTICATIONMODE, CLIENTMODE, ROLLOUTSTATE
 from privacyidea.models import Challenge, db
 from privacyidea.lib.decorators import check_token_locked
 import logging
@@ -68,6 +68,8 @@ import time
 log = logging.getLogger(__name__)
 
 DEFAULT_CHALLENGE_TEXT = _("Please confirm the authentication on your mobile device!")
+ERROR_CHALLENGE_TEXT = _("Use the polling feature of your privacyIDEA Authenticator App"
+                         " to check for a new Login request.")
 DEFAULT_MOBILE_TEXT = _("Do you want to confirm the login?")
 PRIVATE_KEY_SERVER = "private_key_server"
 PUBLIC_KEY_SERVER = "public_key_server"
@@ -80,10 +82,13 @@ DELAY = 1.0
 # Timedelta in minutes
 POLL_TIME_WINDOW = 1
 UPDATE_FB_TOKEN_WINDOW = 5
+POLL_ONLY = "poll only"
 
 
 class PUSH_ACTION(object):
     FIREBASE_CONFIG = "push_firebase_configuration"
+    REGISTRATION_URL = "push_registration_url"
+    TTL = "push_ttl"
     MOBILE_TEXT = "push_text_on_mobile"
     MOBILE_TITLE = "push_title_on_mobile"
     SSL_VERIFY = "push_ssl_verify"
@@ -163,14 +168,14 @@ def create_push_token_url(url=None, ttl=10, issuer="privacyIDEA", serial="mylabe
                                        extra=_construct_extra_parameters(extra_data)))
 
 
-def _build_smartphone_data(serial, challenge, fb_gateway, pem_privkey, options):
+def _build_smartphone_data(serial, challenge, registration_url, pem_privkey, options):
     """
     Create the dictionary to be send to the smartphone as challenge
 
     :param challenge: base32 encoded random data string
     :type challenge: str
-    :param fb_gateway: the gateway object containing the firebase configuration
-    :type fb_gateway: privacyidea.lib.smsprovider.SMSProvider.ISMSProvider
+    :param registration_url: The privacyIDEA URL, to which the Push token communicates
+    :type registration_url: str
     :param options: the options dictionary
     :type options: dict
     :return: the created smartphone_data dictionary
@@ -180,8 +185,6 @@ def _build_smartphone_data(serial, challenge, fb_gateway, pem_privkey, options):
                                                options) or "1"
     sslverify = getParam({"sslverify": sslverify}, "sslverify",
                          allowed_values=["0", "1"], default="1")
-    # We send the challenge to the Firebase service
-    url = fb_gateway.smsgateway.option_dict.get(FIREBASE_CONFIG.REGISTRATION_URL)
     message_on_mobile = get_action_values_from_options(SCOPE.AUTH,
                                                        PUSH_ACTION.MOBILE_TEXT,
                                                        options) or DEFAULT_MOBILE_TEXT
@@ -192,7 +195,7 @@ def _build_smartphone_data(serial, challenge, fb_gateway, pem_privkey, options):
                        "serial": serial,
                        "title": title,
                        "sslverify": sslverify,
-                       "url": url}
+                       "url": registration_url}
     # Create the signature.
     # value to string
     sign_string = u"{nonce}|{url}|{serial}|{question}|{title}|{sslverify}".format(**smartphone_data)
@@ -226,7 +229,7 @@ def _build_verify_object(pubkey_pem):
 
 class PushTokenClass(TokenClass):
     """
-    The :ref:`push_token` uses the firebase service to send challenges to the
+    The :ref:`push_token` uses the Firebase service to send challenges to the
     user's smartphone. The user confirms on the smartphone, signs the
     challenge and sends it back to privacyIDEA.
 
@@ -247,7 +250,7 @@ class PushTokenClass(TokenClass):
 
             enrollment_credential=<hex nonce>
             serial=<token serial>
-            fbtoken=<firebase token>
+            fbtoken=<Firebase token>
             pubkey=<public key>
 
     For more information see:
@@ -255,7 +258,10 @@ class PushTokenClass(TokenClass):
     - https://github.com/privacyidea/privacyidea/issues/1342
     - https://github.com/privacyidea/privacyidea/wiki/concept%3A-PushToken
     """
-    mode = [TOKENMODE.AUTHENTICATE, TOKENMODE.CHALLENGE, TOKENMODE.OUTOFBAND]
+    mode = [AUTHENTICATIONMODE.AUTHENTICATE, AUTHENTICATIONMODE.CHALLENGE, AUTHENTICATIONMODE.OUTOFBAND]
+    client_mode = CLIENTMODE.POLL
+    # A disabled PUSH token has to be removed from the list of checked tokens.
+    check_if_disabled = False
 
     def __init__(self, db_token):
         TokenClass.__init__(self, db_token)
@@ -263,7 +269,6 @@ class PushTokenClass(TokenClass):
         self.mode = ['challenge', 'authenticate']
         self.hKeyRequired = False
 
-
     @staticmethod
     def get_class_type():
         """
@@ -301,7 +306,19 @@ class PushTokenClass(TokenClass):
                            'type': 'str',
                            'desc': _('The configuration of your Firebase application.'),
                            'group': "PUSH",
-                           'value': [gw.identifier for gw in gws]
+                           'value': [POLL_ONLY] + [gw.identifier for gw in gws]
+                       },
+                       PUSH_ACTION.REGISTRATION_URL: {
+                            "required": True,
+                            'type': 'str',
+                            'group': "PUSH",
+                            'desc': _('The URL the Push App should contact in the second enrollment step.'
+                                      ' Usually it is the endpoint /ttype/push of the privacyIDEA server.')
+                       },
+                       PUSH_ACTION.TTL: {
+                           'type': 'int',
+                           'group': "PUSH",
+                           'desc': _('The second enrollment step must be completed within this time (in minutes).')
                        },
                        PUSH_ACTION.SSL_VERIFY: {
                            'type': 'str',
@@ -397,7 +414,7 @@ class PushTokenClass(TokenClass):
 
         if "serial" in upd_param and "fbtoken" in upd_param and "pubkey" in upd_param:
             # We are in step 2:
-            if self.token.rollout_state != "clientwait":
+            if self.token.rollout_state != ROLLOUTSTATE.CLIENTWAIT:
                 raise ParameterError("Invalid state! The token you want to enroll is not in the state 'clientwait'.")
             enrollment_credential = getParam(upd_param, "enrollment_credential", optional=False)
             if enrollment_credential != self.get_tokeninfo("enrollment_credential"):
@@ -416,7 +433,7 @@ class PushTokenClass(TokenClass):
             # We are in step 1:
             upd_param["2stepinit"] = 1
             self.add_tokeninfo("enrollment_credential", geturandom(20, hex=True))
-            # We also store the firebase config, that was used during the enrollment.
+            # We also store the Firebase config, that was used during the enrollment.
             self.add_tokeninfo(PUSH_ACTION.FIREBASE_CONFIG, param.get(PUSH_ACTION.FIREBASE_CONFIG))
         else:
             raise ParameterError("Invalid Parameters. Either provide (genkey) or (serial, fbtoken, pubkey).")
@@ -445,23 +462,29 @@ class PushTokenClass(TokenClass):
         imageurl = params.get("appimageurl")
         if imageurl:
             extra_data.update({"image": imageurl})
-        if self.token.rollout_state == "clientwait":
+        if self.token.rollout_state == ROLLOUTSTATE.CLIENTWAIT:
+            # Get enrollment values from the policy
+            registration_url = getParam(params, PUSH_ACTION.REGISTRATION_URL, optional=False)
+            ttl = getParam(params, PUSH_ACTION.TTL, default="10")
             # Get the values from the configured PUSH config
             fb_identifier = params.get(PUSH_ACTION.FIREBASE_CONFIG)
-            firebase_configs = get_smsgateway(identifier=fb_identifier, gwtype=GWTYPE)
-            if len(firebase_configs) != 1:
-                raise ParameterError("Unknown Firebase configuration!")
-            fb_options = firebase_configs[0].option_dict
-            for k in [FIREBASE_CONFIG.PROJECT_NUMBER, FIREBASE_CONFIG.PROJECT_ID,
-                      FIREBASE_CONFIG.APP_ID, FIREBASE_CONFIG.API_KEY,
-                      FIREBASE_CONFIG.APP_ID_IOS, FIREBASE_CONFIG.API_KEY_IOS]:
-                extra_data[k] = fb_options.get(k)
+            if fb_identifier != POLL_ONLY:
+                # If do not do poll_only, then we load all the Firebase configuration
+                firebase_configs = get_smsgateway(identifier=fb_identifier, gwtype=GWTYPE)
+                if len(firebase_configs) != 1:
+                    raise ParameterError("Unknown Firebase configuration!")
+                fb_options = firebase_configs[0].option_dict
+                for k in [FIREBASE_CONFIG.PROJECT_NUMBER, FIREBASE_CONFIG.PROJECT_ID,
+                          FIREBASE_CONFIG.APP_ID, FIREBASE_CONFIG.API_KEY,
+                          FIREBASE_CONFIG.APP_ID_IOS, FIREBASE_CONFIG.API_KEY_IOS]:
+                    extra_data[k] = fb_options.get(k)
+                self.add_tokeninfo(FIREBASE_CONFIG.PROJECT_ID, fb_options.get(FIREBASE_CONFIG.PROJECT_ID))
             # this allows to upgrade our crypto
             extra_data["v"] = 1
             extra_data["serial"] = self.get_serial()
             extra_data["sslverify"] = sslverify
             # We display this during the first enrollment step!
-            qr_url = create_push_token_url(url=fb_options.get(FIREBASE_CONFIG.REGISTRATION_URL),
+            qr_url = create_push_token_url(url=registration_url,
                                            user=user.login,
                                            realm=user.realm,
                                            serial=self.get_serial(),
@@ -469,12 +492,11 @@ class PushTokenClass(TokenClass):
                                            issuer=tokenissuer,
                                            user_obj=user,
                                            extra_data=extra_data,
-                                           ttl=fb_options.get(FIREBASE_CONFIG.TTL))
+                                           ttl=ttl)
             response_detail["pushurl"] = {"description": _("URL for privacyIDEA Push Token"),
                                           "value": qr_url,
                                           "img": create_img(qr_url, width=250)
                                           }
-            self.add_tokeninfo(FIREBASE_CONFIG.PROJECT_ID, fb_options.get(FIREBASE_CONFIG.PROJECT_ID))
 
             response_detail["enrollment_credential"] = self.get_tokeninfo("enrollment_credential")
 
@@ -532,7 +554,7 @@ class PushTokenClass(TokenClass):
             try:
                 token_obj = get_one_token(serial=serial,
                                           tokentype="push",
-                                          rollout_state="clientwait")
+                                          rollout_state=ROLLOUTSTATE.CLIENTWAIT)
                 token_obj.update(request_data)
             except ResourceNotFoundError:
                 raise ResourceNotFoundError("No token with this serial number "
@@ -646,12 +668,13 @@ class PushTokenClass(TokenClass):
             # The signature was valid now check for an open challenge
             # we need the private server key to sign the smartphone data
             pem_privkey = tok.get_tokeninfo(PRIVATE_KEY_SERVER)
-            # we also need the FirebaseGateway for this token
-            fb_identifier = tok.get_tokeninfo(PUSH_ACTION.FIREBASE_CONFIG)
-            if not fb_identifier:
-                raise ResourceNotFoundError('The pushtoken {0!s} has no Firebase configuration '
-                                            'assigned.'.format(serial))
-            fb_gateway = create_sms_instance(fb_identifier)
+            # We need the registration URL for the challenge
+            registration_url = get_action_values_from_options(
+                SCOPE.ENROLL, PUSH_ACTION.REGISTRATION_URL, options={'g': g})
+            if not registration_url:
+                raise ResourceNotFoundError('There is no registration_url defined for the '
+                                            ' pushtoken {0!s}. You need to define a push_registration_url '
+                                            'in an enrollment policy.'.format(serial))
             options = {'g': g}
             challenges = []
             challengeobject_list = get_challenges(serial=serial)
@@ -662,7 +685,7 @@ class PushTokenClass(TokenClass):
                     # then return the necessary smartphone data to answer
                     # the challenge
                     sp_data = _build_smartphone_data(serial, chal.challenge,
-                                                     fb_gateway, pem_privkey, options)
+                                                     registration_url, pem_privkey, options)
                     challenges.append(sp_data)
             # return the challenges as a list in the result value
             result = challenges
@@ -696,11 +719,11 @@ class PushTokenClass(TokenClass):
               Host: https://yourprivacyideaserver
 
               serial=<token serial>
-              fbtoken=<firebase token>
+              fbtoken=<Firebase token>
               pubkey=<public key>
 
         - It is also used when the smartphone sends the signed response
-          to the challenge during authentication. The following parameters ar accepted:
+          to the challenge during authentication. The following parameters are accepted:
 
             .. sourcecode:: http
 
@@ -721,7 +744,7 @@ class PushTokenClass(TokenClass):
               POST /ttype/push HTTP/1.1
               Host: https://yourprivacyideaserver
 
-              new_fb_token=<new firebase token>
+              new_fb_token=<new Firebase token>
               serial=<token serial>
               timestamp=<timestamp>
               signature=SIGNATURE(<new_fb_token>|<serial>|<timestamp>)
@@ -791,24 +814,29 @@ class PushTokenClass(TokenClass):
         The return tuple builds up like this:
         ``bool`` if submit was successful;
         ``message`` which is displayed in the JSON response;
-        additional ``attributes``, which are displayed in the JSON response.
+        additional challenge ``reply_dict``, which are displayed in the JSON challenges response.
         """
         options = options or {}
         message = get_action_values_from_options(SCOPE.AUTH,
                                                  ACTION.CHALLENGETEXT,
                                                  options) or DEFAULT_CHALLENGE_TEXT
 
-        attributes = None
         data = None
+        # Initially we assume there is no error from Firebase
+        res = True
         fb_identifier = self.get_tokeninfo(PUSH_ACTION.FIREBASE_CONFIG)
         if fb_identifier:
             challenge = b32encode_and_unicode(geturandom())
-            fb_gateway = create_sms_instance(fb_identifier)
-            pem_privkey = self.get_tokeninfo(PRIVATE_KEY_SERVER)
-            smartphone_data = _build_smartphone_data(self.token.serial,
-                                                     challenge, fb_gateway,
-                                                     pem_privkey, options)
-            res = fb_gateway.submit_message(self.get_tokeninfo("firebase_token"), smartphone_data)
+            if fb_identifier != POLL_ONLY:
+                # We only push to Firebase if this tokens does NOT POLL_ONLY.
+                fb_gateway = create_sms_instance(fb_identifier)
+                registration_url = get_action_values_from_options(
+                    SCOPE.ENROLL, PUSH_ACTION.REGISTRATION_URL, options=options)
+                pem_privkey = self.get_tokeninfo(PRIVATE_KEY_SERVER)
+                smartphone_data = _build_smartphone_data(self.token.serial,
+                                                         challenge, registration_url,
+                                                         pem_privkey, options)
+                res = fb_gateway.submit_message(self.get_tokeninfo("firebase_token"), smartphone_data)
 
             # Create the challenge in the challenge table if either the message
             # was successfully submitted to the Firebase API or if polling is
@@ -833,17 +861,25 @@ class PushTokenClass(TokenClass):
                                          validitytime=validity)
                 db_challenge.save()
                 self.challenge_janitor()
+                transactionid = db_challenge.transaction_id
 
-            # If sending the Push message failed, we still raise an error.
+            # If sending the Push message failed, we log a warning
             if not res:
-                raise ValidateError("Failed to submit message to firebase service.")
+                log.warning(u"Failed to submit message to Firebase service for token {0!s}."
+                            .format(self.token.serial))
+                message += " " + ERROR_CHALLENGE_TEXT
+                if is_true(options.get("exception")):
+                    raise ValidateError("Failed to submit message to Firebase service.")
         else:
             log.warning(u"The token {0!s} has no tokeninfo {1!s}. "
                         u"The message could not be sent.".format(self.token.serial,
                                                                  PUSH_ACTION.FIREBASE_CONFIG))
-            raise ValidateError("The token has no tokeninfo. Can not send via firebase service.")
+            message += " " + ERROR_CHALLENGE_TEXT
+            if is_true(options.get("exception")):
+                raise ValidateError("The token has no tokeninfo. Can not send via Firebase service.")
 
-        return True, message, db_challenge.transaction_id, attributes
+        reply_dict = {"attributes": {"hideResponseInput": self.client_mode != CLIENTMODE.INTERACTIVE}}
+        return True, message, transactionid, reply_dict
 
     @check_token_locked
     def authenticate(self, passw, user=None, options=None):
