@@ -63,6 +63,7 @@ from privacyidea.lib.crypto import safe_compare
 from privacyidea.lib.smsprovider.SMSProvider import (get_sms_provider_class,
                                                      create_sms_instance,
                                                      get_smsgateway)
+from privacyidea.lib.tokens.hotptoken import VERIFY_ENROLLMENT_MESSAGE
 from json import loads
 from privacyidea.lib import _
 
@@ -206,8 +207,8 @@ class SmsTokenClass(HotpTokenClass):
                    SCOPE.AUTH: {
                         SMSACTION.SMSTEXT: {
                             'type': 'str',
-                            'desc': _('The text that will be send via SMS for'
-                                      ' an SMS token. Use <otp> and <serial> '
+                            'desc': _('The text that will be send via SMS for '
+                                      'an SMS token. Use tags like {otp} and {serial} '
                                       'as parameters.')},
                         SMSACTION.SMSAUTO: {
                             'type': 'bool',
@@ -269,17 +270,19 @@ class SmsTokenClass(HotpTokenClass):
         :type param: dict
         :return: nothing
         """
-        if getParam(param, "dynamic_phone", optional):
-            self.add_tokeninfo("dynamic_phone", True)
-        else:
-            # specific - phone
-            phone = getParam(param, "phone", required)
-            self.add_tokeninfo("phone", phone)
+        verify = getParam(param, "verify", optional=True)
+        if not verify:
+            if getParam(param, "dynamic_phone", optional):
+                self.add_tokeninfo("dynamic_phone", True)
+            else:
+                # specific - phone
+                phone = getParam(param, "phone", required)
+                self.add_tokeninfo("phone", phone)
 
-        # in case of the sms token, only the server must know the otpkey
-        # thus if none is provided, we let create one (in the TokenClass)
-        if "genkey" not in param and "otpkey" not in param:
-            param['genkey'] = 1
+            # in case of the sms token, only the server must know the otpkey
+            # thus if none is provided, we let create one (in the TokenClass)
+            if "genkey" not in param and "otpkey" not in param:
+                param['genkey'] = 1
 
         HotpTokenClass.update(self, param, reset_failcount)
 
@@ -306,13 +309,12 @@ class SmsTokenClass(HotpTokenClass):
         :param transactionid: the id of this challenge
         :param options: the request context parameters / data
                 You can pass exception=1 to raise an exception, if
-                the SMS could not be sent. Otherwise the message is contained in the response.
+                the SMS could not be sent.
         :return: tuple of (bool, message and data)
                  bool, if submit was successful
                  message is submitted to the user
                  data is preserved in the challenge
-                 attributes - additional attributes, which are displayed in the
-                    output
+                 reply_dict - additional reply_dict, which is added to the response
         """
         success = False
         options = options or {}
@@ -320,7 +322,7 @@ class SmsTokenClass(HotpTokenClass):
                                                         "{0!s}_{1!s}".format(self.get_class_type(),
                                                                              ACTION.CHALLENGETEXT),
                                                         options) or _("Enter the OTP from the SMS:")
-        attributes = {'state': transactionid}
+        reply_dict = {'attributes': {'state': transactionid}}
         validity = self._get_sms_timeout()
 
         if self.is_active() is True:
@@ -349,9 +351,9 @@ class SmsTokenClass(HotpTokenClass):
                 db_challenge.save()
                 transactionid = transactionid or db_challenge.transaction_id
             except Exception as e:
-                info = ("The PIN was correct, but the "
-                        "SMS could not be sent: %r" % e)
-                log.warning(info)
+                info = _("The PIN was correct, but the "
+                         "SMS could not be sent!")
+                log.warning(info + " ({0!s})".format(e))
                 log.debug("{0!s}".format(traceback.format_exc()))
                 return_message = info
                 if is_true(options.get("exception")):
@@ -359,9 +361,9 @@ class SmsTokenClass(HotpTokenClass):
 
         expiry_date = datetime.datetime.now() + \
                                     datetime.timedelta(seconds=validity)
-        attributes['valid_until'] = "{0!s}".format(expiry_date)
+        reply_dict['attributes']['valid_until'] = "{0!s}".format(expiry_date)
 
-        return success, return_message, transactionid, attributes
+        return success, return_message, transactionid, reply_dict
 
     @log_with(log)
     @check_token_locked
@@ -551,3 +553,17 @@ class SmsTokenClass(HotpTokenClass):
             autosms = len(autosmspol) >= 1
 
         return autosms
+
+    def prepare_verify_enrollment(self):
+        """
+        This is called, if the token should be enrolled in a way, that the user
+        needs to provide a proof, that the server can verify, that the token
+        was successfully enrolled.
+        The email token needs to send an email with OTP.
+
+        The returned dictionary is added to the response in "detail" -> "verify".
+
+        :return: A dictionary with information that is needed to trigger the verification.
+        """
+        self.create_challenge()
+        return {"message": VERIFY_ENROLLMENT_MESSAGE}
