@@ -56,6 +56,10 @@ log = logging.getLogger(__name__)
 
 class TotpTokenClass(HotpTokenClass):
 
+    # In contrast to the HOTP the counter does not contain the next OTP value,
+    # but the last used OTP value, so we need to set this to 0.
+    previous_otp_offset = 0
+
     @log_with(log)
     def __init__(self, db_token):
         """
@@ -443,14 +447,17 @@ class TotpTokenClass(HotpTokenClass):
                     counter = int((server_time / self.timestep) + 0.5)
 
                     shift = otp2c - counter
-                    self.add_tokeninfo("timeShift", shift)
+                    info["timeShift"] = shift
+                    self.set_tokeninfo(info)
 
                 # now clean the resync data
-                self.del_tokeninfo("otp1c")
+                del info["otp1c"]
+                self.set_tokeninfo(info)
 
             else:
                 log.debug("setting otp1c: {0!s}".format(res))
-                self.add_tokeninfo("otp1c", res)
+                info["otp1c"] = res
+                self.set_tokeninfo(info)
                 res = -1
 
         return res
