@@ -18,26 +18,17 @@
  * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
-myApp.controller("smtpServerController", function($scope, $stateParams, inform,
-                                                  gettextCatalog, $state,
-                                                  $location, ConfigFactory) {
+myApp.controller("smtpServerController", ["$scope", "$stateParams", "inform",
+                                          "gettextCatalog", "$state",
+                                          "$location", "ConfigFactory",
+                                          function($scope, $stateParams, inform,
+                                                   gettextCatalog, $state,
+                                                   $location, ConfigFactory) {
+    // set the default route
     if ($location.path() === "/config/smtp") {
         $location.path("/config/smtp/list");
     }
 
-    $scope.identifier = $stateParams.identifier;
-    if ($scope.identifier) {
-        // We are editing an existing SMTP Server
-        $scope.getSmtpServers($scope.identifier);
-    } else {
-        // This is a new SMTP server
-        $scope.params = {
-            tls: true,
-            port: 25,
-            timeout: 10
-        }
-    }
-
     // Get all servers
     $scope.getSmtpServers = function (identifier) {
         ConfigFactory.getSmtp(function(data) {
@@ -52,6 +43,24 @@ myApp.controller("smtpServerController", function($scope, $stateParams, inform,
         });
     };
 
+    if ($location.path() == "/config/smtp/list") {
+        // in case of list we fetch all smtp servers
+        $scope.getSmtpServers();
+    }
+
+    $scope.identifier = $stateParams.identifier;
+    if ($scope.identifier) {
+        // We are editing an existing SMTP Server
+        $scope.getSmtpServers($scope.identifier);
+    } else {
+        // This is a new SMTP server
+        $scope.params = {
+            tls: true,
+            port: 25,
+            timeout: 10
+        }
+    }
+
     $scope.delSmtpServer = function (identifier) {
         ConfigFactory.delSmtp(identifier, function(data) {
             $scope.getSmtpServers();
@@ -64,8 +73,6 @@ myApp.controller("smtpServerController", function($scope, $stateParams, inform,
         });
     };
 
-    $scope.getSmtpServers();
-
     $scope.sendTestEmail = function() {
         ConfigFactory.testSmtp($scope.params, function(data) {
            if (data.result.value === true) {
@@ -88,4 +95,4 @@ myApp.controller("smtpServerController", function($scope, $stateParams, inform,
 
     // listen to the reload broadcast
     $scope.$on("piReload", $scope.getSmtpServers);
-});
+}]);
