@@ -27,16 +27,22 @@ String.prototype.mysplit = function(separator) {
 
 
 angular.module("privacyideaApp")
-    .controller("mainController",
-                            function (Idle,
-                                      $scope, $http, $location,
-                                      authUrl, AuthFactory, $rootScope,
-                                      $state, ConfigFactory, inform,
-                                      PolicyTemplateFactory, gettextCatalog,
-                                      hotkeys, RegisterFactory,
-                                      U2fFactory, webAuthnToken, instanceUrl,
-                                      PollingAuthFactory, $transitions,
-                                      resourceNamePatterns) {
+    .controller("mainController", ["Idle", "$scope", "$sce", "$http", "$location",
+                                   "authUrl", "validateUrl", "AuthFactory", "$rootScope",
+                                   "$state", "ConfigFactory", "inform",
+                                   "PolicyTemplateFactory", "gettextCatalog",
+                                   "hotkeys", "RegisterFactory",
+                                   "U2fFactory", "webAuthnToken", "instanceUrl",
+                                   "PollingAuthFactory", "$transitions",
+                                   "resourceNamePatterns",
+                                   function (Idle, $scope, $sce, $http, $location,
+                                             authUrl, validateUrl, AuthFactory, $rootScope,
+                                             $state, ConfigFactory, inform,
+                                             PolicyTemplateFactory, gettextCatalog,
+                                             hotkeys, RegisterFactory,
+                                             U2fFactory, webAuthnToken, instanceUrl,
+                                             PollingAuthFactory, $transitions,
+                                             resourceNamePatterns) {
 
     $scope.instanceUrl = instanceUrl;
     $scope.checkRight = AuthFactory.checkRight;
@@ -44,11 +50,14 @@ angular.module("privacyideaApp")
     $scope.checkMainMenu = AuthFactory.checkMainMenu;
     $scope.checkEnroll = AuthFactory.checkEnroll;
     $scope.inputNamePatterns = resourceNamePatterns;
+    $scope.startRoute = "/token";
     var obj = angular.element(document.querySelector("#REMOTE_USER"));
     $scope.remoteUser = obj.val();
     if (!$scope.remoteUser) {
         $scope.loginWithCredentials = true;
     }
+    obj = angular.element(document.querySelector('#FORCE_REMOTE_USER'));
+    $scope.forceRemoteUser = obj.val();
     obj = angular.element(document.querySelector("#PASSWORD_RESET"));
     $scope.passwordReset = obj.val();
     obj = angular.element(document.querySelector("#HSM_READY"));
@@ -65,7 +74,18 @@ angular.module("privacyideaApp")
     obj = angular.element(document.querySelector('#HAS_JOB_QUEUE'));
     $scope.hasJobQueue = obj.val() == "True";
     obj = angular.element(document.querySelector('#LOGIN_TEXT'));
-    $scope.piLoginText= obj.val();
+    $scope.piLoginText = obj.val();
+    obj = angular.element(document.querySelector('#SHOW_NODE'));
+    $scope.show_node = obj.val();
+    obj = angular.element(document.querySelector('#GDPR_LINK'));
+    // we need to trust the GDPR URI explicitly since an admin can change this via policy.
+    $scope.piGDPRLink = $sce.trustAsUrl(obj.val());
+    obj = angular.element(document.querySelector('#PI_TRANSLATION_WARNING'));
+    $scope.piTranslationWarning = obj.val() !== "False";
+    $scope.piTranslationPrefix = obj.val();
+    gettextCatalog.debug = $scope.piTranslationWarning;
+    gettextCatalog.debugPrefix = $scope.piTranslationPrefix;
+
     // Check if registration is allowed
     $scope.registrationAllowed = false;
     RegisterFactory.status(function (data) {
@@ -238,26 +258,27 @@ angular.module("privacyideaApp")
                     if (multi_challenge.length > 1) {
                         $scope.challenge_message = $scope.challenge_message + ' ' + multi_challenge[i].serial;
                     }
-                    var attributes = multi_challenge[i].attributes;
-                    if (attributes === null || attributes.hideResponseInput !== true) {
+                    let challenge = multi_challenge[i];
+                    let attributes = challenge.attributes ? challenge.attributes : null;
+                    if (challenge === null || (attributes && attributes.hideResponseInput !== true)) {
                         $scope.hideResponseInput = false;
                     }
-                    if (attributes !== null) {
-                        if (attributes.u2fSignRequest) {
+                    if (challenge !== null) {
+                        if (attributes && attributes.u2fSignRequest) {
                            $scope.u2fSignRequests.push(attributes.u2fSignRequest);
                         }
-                        if (attributes.webAuthnSignRequest) {
+                        if (attributes && attributes.webAuthnSignRequest) {
                             $scope.webAuthnSignRequests.push(attributes.webAuthnSignRequest);
                         }
-                        if (attributes.img) {
-                            $scope.image = attributes.img;
+                        if (challenge.image) {
+                            $scope.image = challenge.image;
                             if ($scope.image.indexOf("data:image") === -1) {
                                 // In case of an Image link, we prepend the instanceUrl
                                 $scope.image = $scope.instanceUrl + "/" + $scope.image;
                             }
                         }
-                        if (attributes.poll) {
-                            $scope.polling = attributes.poll;
+                        if (challenge.client_mode && challenge.client_mode === 'poll') {
+                            $scope.polling = true;
                         }
                     }
                 }
@@ -331,15 +352,29 @@ angular.module("privacyideaApp")
         // authentication was performed successfully in the background
         // This is used for the TiQR token.
         //debug: console.log("calling check_authentication.");
-        $http.post(authUrl, {
-            username: $scope.login.username,
-            password: "",
-            transaction_id: $scope.transactionid
-        }, {
-            withCredentials: true
+        $http.get(validateUrl + "/polltransaction", {
+            params: {
+                'transaction_id': $scope.transactionid
+            }
         }).then(function (response) {
-            $scope.do_login_stuff(response.data);
-            PollingAuthFactory.stop();
+            if (response.data.result.value === true) {
+                $http.post(authUrl, {
+                    username: $scope.login.username,
+                    password: "",
+                    transaction_id: $scope.transactionid
+                }).then(function (response) {
+                    $scope.do_login_stuff(response.data);
+                }, function (response) {
+                    console.log('Authentication failed after polling!');
+                    console.log(response.data);
+                });
+                PollingAuthFactory.stop();
+            }
+            // if result.value is false, the challenge hasn't been answered yet.
+            // Continue polling
+        }, function(response) {
+            // the /validate/polltransaction endpoint returned an error
+            console.warn("Polling for transactions returned an error: " + response.data);
         });
     };
 
@@ -374,22 +409,46 @@ angular.module("privacyideaApp")
             if ($scope.dialogNoToken) {
                 $('#dialogNoToken').modal("show");
             }
-            $scope.qr_image_android = data.result.value.qr_image_android || "";
-            $scope.qr_image_ios = data.result.value.qr_image_ios || "";
-            $scope.qr_image_custom = data.result.value.qr_image_custom || "";
-            // This is a helper variable to get the number of qr images to display
-            $scope.qr_image_count = Number($scope.qr_image_android.length > 0)
-                + Number($scope.qr_image_ios.length > 0)
-                + Number($scope.qr_image_custom.length > 0);
+            $scope.qr_images = [];
+            if ( data.result.value.qr_image_android ) {
+                $scope.qr_images.push({
+                    'src': data.result.value.qr_image_android,
+                    'alt': 'QR-Code with link to android app in play store',
+                    'help': gettextCatalog.getString('Get the Authenticator App for Android.')})
+            }
+            if ( data.result.value.qr_image_ios ) {
+                $scope.qr_images.push({
+                    'src': data.result.value.qr_image_ios,
+                    'alt': 'QR-Code with link to iOS app in app store',
+                    'help': gettextCatalog.getString('Get the Authenticator App for iOS.')})
+            }
+            if ( data.result.value.qr_image_custom ) {
+                $scope.qr_images.push({
+                    'src': data.result.value.qr_image_custom,
+                    'alt': 'QR-Code with link to a custom app',
+                    'help': gettextCatalog.getString('Get the Authenticator App.')})
+            }
+            if ($scope.qr_images.length > 0) {
+                $scope.qr_col_md = "col-md-" + parseInt(12 / $scope.qr_images.length);
+            }
             $scope.token_page_size = data.result.value.token_page_size;
             $scope.user_page_size = data.result.value.user_page_size;
             $scope.user_details_in_tokenlist = data.result.value.user_details;
             $scope.default_tokentype = data.result.value.default_tokentype;
             $scope.timeout_action = data.result.value.timeout_action;
+            $scope.admin_dashboard = data.result.value.admin_dashboard;
+            if ($scope.admin_dashboard) {
+                $scope.startRoute = "/dashboard";
+            } else {
+                $scope.startRoute = "/token";
+            }
             $scope.hide_welcome = data.result.value.hide_welcome;
             $scope.hide_buttons = data.result.value.hide_buttons;
             $scope.show_seed = data.result.value.show_seed;
+            $scope.show_node = data.result.value.show_node;
+            $scope.token_rollover = data.result.value.token_rollover;
             $scope.subscription_state = data.result.value.subscription_status;
+            $scope.subscription_state_push = data.result.value.subscription_status_push;
             $rootScope.search_on_enter = data.result.value.search_on_enter;
             // Token specific settings
             $scope.tokensettings = {indexedsecret:
@@ -407,11 +466,57 @@ angular.module("privacyideaApp")
             Idle.watch();
             //debug: console.log("successfully authenticated");
             //debug: console.log($scope.loggedInUser);
+            if ($scope.loggedInUser.role === "admin") {
+            /*
+            * Functions to check and to create a default realm.
+            */
+                ConfigFactory.getRealms(function (data) {
+                    // Check if there is a realm defined, or if we should display the
+                    // Auto Create Dialog
+                    var number_of_realms = Object.keys(data.result.value).length;
+                    if (number_of_realms === 0) {
+                        $('#dialogAutoCreateRealm').modal();
+                    }
+                });
+                /*
+                 Welcome dialog, which displays a lot of information to the
+                 administrator.
+
+                 We display it if
+                 subscription_state = 0 and hide_welcome = false
+                 subscription_state = 1
+                 subscription_state = 2
+                 */
+                $scope.resetWelcome();
+                if (($scope.welcomeStep < 4 && !$scope.hide_welcome) || $scope.welcomeStep < 5) {
+                    // We did not walk through the welcome dialog, yet.
+                    if (($scope.subscription_state === 0 && !$scope.hide_welcome) ||
+                        ($scope.subscription_state === 1)) {
+                        $('#dialogWelcome').modal("show");
+                    }
+                }
+                var $random_number = Math.floor(Math.random() * (9999 + 1000 + 1)) + 1000;
+                $scope.class_subscription_expired = "subscriptionExpired" + $random_number;
+                // Show info about privacyIDEA Authenticator App and push
+                if ($scope.subscription_state_push === 1 && !$scope.hide_welcome) {
+                    // no subscription at all
+                     inform.add(gettextCatalog.getString("You are using a certain amount of Push tokens. " +
+                         "Note, that you need a valid subscription, to use Push tokens in push mode. Otherwise" +
+                         " Push tokens will only work in poll only mode. "),
+                            {type: "danger", ttl: 30000});
+                }
+                if ($scope.subscription_state_push === 2) {
+                    // Subscription expired
+                    inform.add(gettextCatalog.getString("Your subscription for the privacyIDEA Authenticator" +
+                        " has expired."),
+                        {type: "danger", ttl: 20000});
+                }
+            }
             if ( $scope.unlocking ) {
                 $('#dialogLock').modal('hide');
             } else {
-                // if we are unlocking we do NOT go to the tokens
-                $location.path("/token");
+                // if we login anew, we either go to the token or to the dashboard
+                $location.path($scope.startRoute);
             }
 
             //inform.add(gettextCatalog.getString("privacyIDEA UI supports " +
@@ -426,7 +531,7 @@ angular.module("privacyideaApp")
         $scope.privacyideaVersionNumber = null;
         $scope.logoutWarning = false;
         $scope.myCountdown = "";
-        $scope.welcomeStep = 0;
+        $scope.resetWelcome();
         $scope.dialogNoToken = false;
         $scope.privacyideaSupportLink = $rootScope.publicLink;
         $state.go("login");
@@ -437,12 +542,18 @@ angular.module("privacyideaApp")
 
     $scope.nextWelcome = function() {
         $scope.welcomeStep += 1;
-        if ($scope.welcomeStep === 4) {
+        if (($scope.subscription_state == 0 && $scope.welcomeStep === 4) ||
+            ($scope.subscription_state == 1 && $scope.welcomeStep === 5)) {
             $('#dialogWelcome').modal("hide");
         }
     };
     $scope.resetWelcome = function() {
-        $scope.welcomeStep = 0;
+        if ($scope.hide_welcome) {
+            $scope.welcomeStep = 4;
+        } else {
+            $scope.welcomeStep = 0;
+        }
+
     };
 
     $scope.closeNoToken = function() {
@@ -453,7 +564,7 @@ angular.module("privacyideaApp")
     $scope.lock_screen = function () {
         // We need to destroy the auth_token
         $scope.loggedInUser.auth_token = null;
-        $scope.welcomeStep = 0;
+        $scope.resetWelcome();
         Idle.unwatch();
         $('#dialogLock').modal({
             keyboard: false,
@@ -505,18 +616,23 @@ angular.module("privacyideaApp")
         $scope.$broadcast("piReload");
     };
 
-});
+}]);
 
 angular.module("privacyideaApp")
-    .controller("pinChangeController",
-                            function (Idle,
-                                      $scope, $http, $location,
-                                      authUrl, AuthFactory, $rootScope,
-                                      $state, ConfigFactory, inform,
-                                      PolicyTemplateFactory, gettextCatalog,
-                                      hotkeys, RegisterFactory,
-                                      U2fFactory, instanceUrl,
-                                      PollingAuthFactory, TokenFactory)
+    .controller("pinChangeController", ["Idle", "$scope", "$http", "$location",
+                                        "authUrl", "AuthFactory", "$rootScope",
+                                        "$state", "ConfigFactory", "inform",
+                                        "PolicyTemplateFactory", "gettextCatalog",
+                                        "hotkeys", "RegisterFactory",
+                                        "U2fFactory", "instanceUrl",
+                                        "PollingAuthFactory", "TokenFactory",
+                                        function (Idle, $scope, $http, $location,
+                                                  authUrl, AuthFactory, $rootScope,
+                                                  $state, ConfigFactory, inform,
+                                                  PolicyTemplateFactory, gettextCatalog,
+                                                  hotkeys, RegisterFactory,
+                                                  U2fFactory, instanceUrl,
+                                                  PollingAuthFactory, TokenFactory)
 {
 
     $scope.newpin = "";
@@ -536,4 +652,4 @@ angular.module("privacyideaApp")
         $scope.logout();
     }
 
-});
+}]);
