@@ -45,15 +45,19 @@ function string_to_date_object(s) {
     return date_obj;
 }
 
-myApp.controller("tokenDetailController", function ($scope,
-                                                    TokenFactory, UserFactory,
-                                                    $stateParams,
-                                                    $state, $rootScope,
-                                                    ValidateFactory,
-                                                    AuthFactory,
-                                                    ConfigFactory,
-                                                    MachineFactory, inform,
-                                                    gettextCatalog) {
+myApp.controller("tokenDetailController", ['$scope', 'TokenFactory',
+                                           'UserFactory', '$stateParams',
+                                           '$state', '$rootScope',
+                                           'ValidateFactory', 'AuthFactory',
+                                           'ConfigFactory', 'MachineFactory',
+                                           'inform', 'gettextCatalog',
+                                           function ($scope, TokenFactory,
+                                                     UserFactory, $stateParams,
+                                                     $state, $rootScope,
+                                                     ValidateFactory,
+                                                     AuthFactory, ConfigFactory,
+                                                     MachineFactory, inform,
+                                                     gettextCatalog) {
     $scope.tokenSerial = $stateParams.tokenSerial;
     // This is the parents object
     $scope.selectedToken.serial = $scope.tokenSerial;
@@ -266,6 +270,27 @@ myApp.controller("tokenDetailController", function ($scope,
         });
     };
 
+    $scope.sendVerifyResponse = function () {
+        var params = {
+            "serial": $scope.token.serial,
+            "verify": $scope.token.verifyResponse,
+            "type": $scope.token.tokentype
+        };
+        TokenFactory.enroll($scope.newUser, params, function (data) {
+            $scope.token.verifyResponse = "";
+            if (data.result.value === true) {
+                inform.add(gettextCatalog.getString("Enrollment successfully verified."),
+                                {type: "info", ttl: 10000});
+            } else {
+                inform.add(gettextCatalog.getString("Enrollment verification failed."),
+                                {type: "danger", ttl: 10000});
+            }
+            $scope.get();
+        });
+    };
+
+
+
     $scope.testOtp = function (otponly) {
         var params = {
             serial: $scope.tokenSerial,
@@ -309,8 +334,14 @@ myApp.controller("tokenDetailController", function ($scope,
             var machineObject = fixMachine($scope.newMachine);
             params["serial"] = $scope.tokenSerial;
             params["application"] = $scope.form.application;
-            params["machineid"] = machineObject.id;
-            params["resolver"] = machineObject.resolver;
+            if ($scope.form.application === "offline") {
+                // We force to be not attached to a machine
+                params["machineid"] = 0;
+                params["resolver"] = "";
+            } else {
+                params["machineid"] = machineObject.id;
+                params["resolver"] = machineObject.resolver;
+            }
             MachineFactory.attachTokenMachine(params, function (data) {
                 // clear form
                 $scope.form.application = null;
@@ -386,5 +417,15 @@ myApp.controller("tokenDetailController", function ($scope,
     // listen to the reload broadcast
     $scope.$on("piReload", $scope.get);
 
+    $scope.rolloverTokenAllowed = function(token) {
+        if ( typeof(token) != 'undefined' ) {
+            if ($scope.checkEnroll() && (token.tokentype in $scope.token_rollover) &&
+                token.info.tokenkind === 'software') {
+                return true;
+            }
+        }
+        return false;
+    };
 
-});
+
+}]);
