@@ -1,9 +1,10 @@
-import json
+# -*- coding: utf-8 -*-
+
 from .base import MyApiTestCase
 from privacyidea.lib.monitoringstats import write_stats
 from privacyidea.lib.tokenclass import AUTH_DATE_FORMAT
+from privacyidea.models import db
 import datetime
-from flask import current_app
 
 
 class APIMonitoringTestCase(MyApiTestCase):
@@ -13,13 +14,13 @@ class APIMonitoringTestCase(MyApiTestCase):
         # create some statistics
 
         write_stats("key1", 1)
-        write_stats("key2", "A")
+        write_stats("key2", 50)
         write_stats("key1", 2)
         ts = datetime.datetime.now().isoformat()
 
-        write_stats("key2", "B")
+        write_stats("key2", 60)
         write_stats("key1", 3)
-        write_stats("key2", "A")
+        write_stats("key2", 50)
         write_stats("key1", 4)
 
         # get available stats keys
@@ -28,10 +29,10 @@ class APIMonitoringTestCase(MyApiTestCase):
                                            headers={'Authorization': self.at}):
 
             res = self.app.full_dispatch_request()
-            self.assertTrue(res.status_code == 200, res)
+            self.assertEqual(200, res.status_code, res)
             result = res.json.get("result")
-            self.assertTrue("key1" in result.get("value"))
-            self.assertTrue("key2" in result.get("value"))
+            self.assertIn("key1", result.get("value"), result)
+            self.assertIn("key2", result.get("value"), result)
 
         # check values of key1
         with self.app.test_request_context('/monitoring/key1',
@@ -91,12 +92,12 @@ class APIMonitoringTestCase(MyApiTestCase):
             res = self.app.full_dispatch_request()
             self.assertTrue(res.status_code == 200, res)
             result = res.json.get("result")
-            self.assertEqual(result.get("value"), 4)
+            self.assertEqual(4, result.get("value"), result)
 
     def test_02_delete_stats(self):
 
         ts = datetime.datetime.now()
-        write_stats("key2", "B")
+        write_stats("key2", 60)
 
         # Now we delete some keys (the three old ones)
         with self.app.test_request_context('/monitoring/key2',
@@ -118,4 +119,4 @@ class APIMonitoringTestCase(MyApiTestCase):
             self.assertTrue(res.status_code == 200, res)
             result = res.json.get("result")
             # Number of remaining values
-            self.assertEqual(len(result.get("value")), 1)
+            self.assertEqual(1, len(result.get("value")), result)
