@@ -58,6 +58,7 @@ class APIPeriodicTasksTestCase(MyApiTestCase):
                 'interval': '0 8 * * *',
                 'taskmodule': 'UnitTest',
                 'ordering': 5,
+                'retry_if_failed': False,
                 'options': '{"something": 123, "else": true}',
             }
             status_code, data = self.simulate_request('/periodictask/', method='POST', data=task_dict1)
@@ -166,6 +167,7 @@ class APIPeriodicTasksTestCase(MyApiTestCase):
         self.assertEqual(result_dict['nodes'], ['pinode1', 'pinode2'])
         self.assertEqual(result_dict['last_runs'], {})
         last_update = parse_timestamp(result_dict['last_update'])
+        self.assertEqual(result_dict['retry_if_failed'], False)
         self.assertIsNotNone(last_update)
         self.assertEqual(result_dict['options'], {'something': '123', 'else': 'True'})
 
@@ -197,8 +199,8 @@ class APIPeriodicTasksTestCase(MyApiTestCase):
         self.assertEqual(status_code, 200)
         self.assertTrue(data['result']['status'])
         self.assertEqual(len(data['result']['value']), 2)
-        self.assertEqual([task['name'] for task in data['result']['value']],
-                         ['new name', 'some other task'])
+        self.assertIn('new name', [task['name'] for task in data['result']['value']], data)
+        self.assertIn('some other task', [task['name'] for task in data['result']['value']], data)
 
         # get updated task
         status_code, data = self.simulate_request('/periodictask/{}'.format(ptask_id1), method='GET')
