@@ -17,6 +17,7 @@ from privacyidea.models import (Token,
 from privacyidea.lib.config import (set_privacyidea_config, set_prepend_pin)
 import datetime
 import binascii
+import time
 
 
 class TOTPTokenTestCase(MyTestCase):
@@ -294,7 +295,7 @@ class TOTPTokenTestCase(MyTestCase):
                                                                    30})
         # Found the counter 47251647
         self.assertTrue(res == 47251647, res)
-        
+
     def test_14_split_pin_pass(self):
         db_token = Token.query.filter_by(serial=self.serial1).first()
         token = TotpTokenClass(db_token)
@@ -562,17 +563,14 @@ class TOTPTokenTestCase(MyTestCase):
                       "otplen": 6})
         token.token.count = 47251640
         token.set_sync_window(10)
-        ts1 = token.get_tokeninfo("timeShift")
         # counter = 47251649 => otp = 705493, is out of sync
         r = token.check_otp(anOtpVal="705493", window=30,
                             options={"initTime": 47251644 * 30})
         self.assertTrue(r == -1, r)
-        ts2 = token.get_tokeninfo("timeShift")
         # counter = 47251650 => otp = 389836, will be autosynced.
         r = token.check_otp(anOtpVal="589836", window=30,
                             options={"initTime": 47251645 * 30 })
         self.assertTrue(r == 47251650, r)
-        ts3 = token.get_tokeninfo("timeShift")
 
         # counter = 47251640 => otp = 166325 is an old OTP value
         # counter = 47251641 => otp = 432730 is an old OTP value
@@ -580,20 +578,16 @@ class TOTPTokenTestCase(MyTestCase):
         r = token.check_otp(anOtpVal="166325", window=30,
                             options={"initTime": 47251644 * 30})
         self.assertTrue(r == -1, r)
-        ts1 = token.get_tokeninfo("timeShift")
         r = token.check_otp(anOtpVal="166325", window=30,
                             options={"initTime": 47251645 * 30})
         self.assertTrue(r == -1, r)
-        ts2 = token.get_tokeninfo("timeShift")
         # Autoresync with two consecutive old OTP values must not work out!
         r = token.check_otp(anOtpVal="166325", window=30,
                             options={"initTime": 47251644 * 30})
         self.assertTrue(r == -1, r)
-        ts3 = token.get_tokeninfo("timeShift")
         r = token.check_otp(anOtpVal="432730", window=30,
                             options={"initTime": 47251645 * 30})
         self.assertTrue(r == -1, r)
-        ts4 = token.get_tokeninfo("timeShift")
 
         # Autosync with a gap in the next otp value will fail
         token.token.count = 47251640
@@ -734,6 +728,39 @@ class TOTPTokenTestCase(MyTestCase):
         r = TotpTokenClass.get_setting_type("totp.blabla")
         self.assertEqual(r, "")
 
+    def test_26_is_previous_otp(self):
+        # check if the OTP was used previously
+        serial = "previous"
+        db_token = Token(serial, tokentype="totp")
+        db_token.save()
+        token = TotpTokenClass(db_token)
+        token.set_otpkey(self.otpkey)
+        token.set_hashlib("sha1")
+        token.set_otplen(6)
+        # Authenticate with the current OTP value
+        counter = token._time2counter(time.time(), timeStepping=30)
+        otp_now = token._calc_otp(counter)
+        r = token.check_otp(otp_now, window=180)
+        self.assertEqual(r, counter)
+        # Now we try several is_previous_otp and the timeShift must stay the same!
+        ts0 = float(token.get_tokeninfo("timeShift"))
+        self.assertTrue(-181 < ts0 < 181)
+        # Too old
+        r = token.is_previous_otp(token._calc_otp(counter - 3))
+        self.assertEqual(r, False)
+        ts = float(token.get_tokeninfo("timeShift"))
+        self.assertEqual(ts, ts0)
+        # The same OTP value
+        r = token.is_previous_otp(otp_now)
+        self.assertEqual(r, True)
+        ts = float(token.get_tokeninfo("timeShift"))
+        self.assertEqual(ts, ts0)
+        # Future value
+        r = token.is_previous_otp(token._calc_otp(counter + 8))
+        self.assertEqual(r, False)
+        ts = float(token.get_tokeninfo("timeShift"))
+        self.assertEqual(ts, ts0)
+
     def test_27_get_default_settings(self):
         params = {}
         g = FakeFlaskG()
